package uprrp.tania.networking;

import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

import javax.net.ssl.HttpsURLConnection;

import uprrp.tania.utils.URLEventListener;

public class SendConsentForm extends AsyncTask <String, Void, String> {

    private static final String TAG = "SendConsentForm";
    private static final String consentFormBaseURL = "https://tania.uprrp.edu/getSignatureIOS.php";
    private final URLEventListener myCallback;

    public SendConsentForm(URLEventListener callback) {
        this.myCallback = callback;
    }

    private boolean validInputs(String... strings) {

        if(strings.length != 3) {
            Log.e(TAG, "Invalid string array length!");
            return false;
        }

        for(int i = 0; i < strings.length; i++) {
            if(strings[i] == null) {
                Log.e(TAG, "Encountered null parameter on index " + i);
                return false;
            }
        }

        return true;

    }

    // TODO: instead of making a normal POST request with
    //  the body data=<JSON>, we should just send the JSON itself
    //  This is adding unnecessary layers of complexity when decoding AND encoding
    /*
        JSON format:
        {
            "signatureDate": String,
            "signaturePDFBase64": String,
            "token": String,
        }
    */
    @Override
    protected String doInBackground(String... strings) {

        // Validation
        if(!validInputs(strings)) {
            Log.e(TAG, "Invalid inputs given!");
            return null;
        }

        // Extract variables
        String deviceToken = strings[0];
        String consentFormBase64 = strings[1];
        String signatureDate = strings[2];

        try {

            // Create JSON
            JSONObject consentFormJSON = new JSONObject();
            consentFormJSON.put("token", deviceToken);
            consentFormJSON.put("signatureDate", signatureDate);
            consentFormJSON.put("signaturePDFBase64", consentFormBase64);
            Log.d(TAG, "ConsentFormJSON is " + consentFormJSON.toString()); // log

            // Encode data
            String encodedRequestBody = URLEncoder.encode("data", "UTF-8") + "=" + URLEncoder.encode(consentFormJSON.toString(), "UTF-8");

            // Send POST data request
            URL url = new URL(consentFormBaseURL);
            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
            conn.setDoOutput(true);
            Writer writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(), StandardCharsets.UTF_8));
            writer.write(encodedRequestBody);
            writer.close();

            // Get the server response
            BufferedReader serverReader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            StringBuilder serverResponse = new StringBuilder();

            String line = "";
            while(line != null) {
                serverResponse.append(line);
                line = serverReader.readLine();
            }

            String response = serverResponse.toString();
            Log.d(TAG, "The server's response is " + response);
            if(response.toLowerCase().contains("error")) {
                return null;
            } else {
                return response;
            }

        } catch (JSONException e) {
            Log.e(TAG, "Couldn't prepare consentFormJSON!");
            e.printStackTrace();
            return null;
        } catch (UnsupportedEncodingException e) {
            Log.e(TAG, "Couldn't encode consentFormJSON!");
            e.printStackTrace();
            return null;
        } catch(Exception e) {
            Log.e(TAG, "Couldn't communicate with server while recovering account!");
            e.printStackTrace();
            return null;
        }

    }

    @Override
    protected void onPostExecute(String response) {

        if(this.myCallback == null) {
            Log.e(TAG, "Callback wasn't initialized first!");
            return;
        }

        if(response == null) {
            this.myCallback.onFailure(new Exception("Error occurred during transaction!"));
            return;
        }

        this.myCallback.onSuccess();

    }

}
